/*
** Module   : TempTableBuilder.java
** Abstract : Implementation of the dynamic features for constructing temp-tables  
*              (attributes, methods and constructors).
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 OM  20130129 Created initial version.
** 002 VMN 20130217 Added support for handle-based attributes:
**                  ERROR (KW_ERROR)
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Implementation of buffer of a dynamic TEMP-TABLE and the associated 
 * attributes and methods.
 * This {@link TempTable} interface is needed by {@link CommonHandle}
 * in order to store the handle to a the table.
 * 
 * @author om
 */
public abstract class TempTableBuilder
implements TempTable
{
   /**
    * Dynamically creates a temporary table at runtime and assign it to a 
    * handle. The table is empty and will be populated with fileds, indexes 
    * and relations subsequent statements.
    * This method is the P2J equivalent of <code>CREATE TEMP-TABLE</code> 
    * statement of Progress 4GL.
    * 
    * @param   tth
    *          The handle to assign the newly created temporary table to.
    * @param   widgetPool
    *          The name of the widget pool that contains the new temp table, 
    *          if any, or empty string to create the object in the current 
    *          default widget pool.
    */
   public static void create(handle tth, String widgetPool)
   {
      // TODO Auto-generated method stub
   }

   /**
    * Dynamically creates a temporary table at runtime and assign it to a 
    * handle. The table is empty and will be populated with fileds, indexes 
    * and relations subsequent statements.
    * This method is the P2J equivalent of <code>CREATE TEMP-TABLE</code> 
    * statement of Progress 4GL.
    * 
    * @param   tth
    *          The handle to assign the newly created temporary table to.
    * @param   widgetPool
    *          The name of the widget pool that contains the new temp table, 
    *          if any, or empty string to create the object in the current 
    *          default widget pool.
    */
   public static void create(handle tth, character widgetPool)
   {
      if (widgetPool.isUnknown())
      {
         TempTableBuilder.create(tth, "");
      }
      else
      {
         TempTableBuilder.create(tth, widgetPool.getValue());
      }
   }

   /**
    * Dynamically creates a temporary table at runtime and assign it to a 
    * handle. The table is empty and will be populated with fileds, indexes 
    * and relations subsequent statements.
    * This method is the P2J equivalent of <code>CREATE TEMP-TABLE</code> 
    * statement of Progress 4GL.
    *
    * @param   tth
    *          The handle to assign the newly created temporary table to.
    */
   public static void create(handle tth)
   {
      TempTableBuilder.create(tth, "");
   }

   /**
    * Duplicates the structure of an existing table, copying the field 
    * definitions, and optionally, the indexes.
    * This method is the P2J equivalent of <code>CREATE-LIKE</code> method
    * of Progress 4GL.
    *
    * @param   hbuf
    *          A handler assigned to a table buffer to create the copy.
    *
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical createTableLike(handle hbuf)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds a new field to the temp-table. This overloaded form creates field  
    * with properties se to their default values.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    *
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(String name, String type)
   {
      return addNewField(name, type, 0, "", "", "", "");
   }

   /**
    * Adds a field with the specified properties to the temp-table.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    * @param   extent
    *          An integer expression specifying the extent of an array.
    * @param   format
    *          The data format for the defined data type. If null or unknown, 
    *          the default format will be used.
    * @param   initial
    *          An expression that evaluates to the initial value of the 
    *          defined field. 
    *          TODO: this method will probably be overloaded because of this.
    * @param   label
    *          The label of the defined field. If null or unknown the name 
    *          parameter will be used.
    * @param   columnLabel
    *          The label of the column associated with the defined field
    *          
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(String name,
                              String type,
                              int    extent,
                              String format,
                              String initial,
                              String label,
                              String columnLabel)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds a field with the specified properties to the temp-table.
    * This method is the P2J equivalent of <code>ADD-NEW-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   type
    *          The data type of the specified field.
    * @param   extent
    *          An integer expression specifying the extent of an array.
    * @param   format
    *          The data format for the defined data type. If null or unknown, 
    *          the default format will be used.
    * @param   initial
    *          An expression that evaluates to the initial value of the 
    *          defined field. 
    *          TODO: this method will probably be overloaded because of this.
    * @param   label
    *          The label of the defined field. If null or unknown the name 
    *          parameter will be used.
    * @param   columnLabel
    *          The label of the column associated with the defined field
    *          
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewField(character    name,
                              character    type,
                              int          extent,
                              character    format,
                              BaseDataType initial,
                              character    label,
                              character    columnLabel)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds a new index with the specified name to the temp-table. Newly 
    * created idex is empty and its components must be added individually 
    * with the <code>ADD-INDEX-FIELD()</code> method.
    * This form of the overloaded method creates an index that is not unique, 
    * not primary and not a word index.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to be created.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewIndex(String name)
   {
      return addNewIndex(name, false, false, false);
   }

   /**
    * Adds a new index with the specified name to the temp-table. Newly 
    * created idex is empty and its components must be added individually 
    * with the <code>ADD-INDEX-FIELD()</code> method.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the index to be created.
    * @param   unique
    *          TRUE if this index is unique.
    * @param   primary
    *          TRUE if this is the primary index.
    * @param   word
    *          TRUE if this is a word index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addNewIndex(String  name,
                              boolean unique,
                              boolean primary,
                              boolean word)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds a new empty index with the specified name to the temp-table. Index 
    * components must be added with the <code>ADD-INDEX-FIELD()</code> method.
    * This method is the P2J equivalent of <code>ADD-NEW-INDEX</code> method
    * of Progress 4GL.
    *
    * @param   name
    *          The name of the index to be created.
    * @param   unique
    *          TRUE if this index is unique.
    * @param   primary
    *          TRUE if this is the primary index.
    * @param   word
    *          TRUE if this is a word index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical addNewIndex(String  name,
                              logical unique,
                              logical primary,
                              logical word)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Database field name from which to copy the field. The table 
    *          name must be qualified with the database name.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(String name, String source)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Database field name from which to copy the field. The table 
    *          name must be qualified with the database name.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(character name, character source)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds to the temp-table a field like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the field to be created in the temp-table.
    * @param   source
    *          Handle to field name from which to copy the field.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addFieldLike(character name, handle source)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Adds to the temp-table an index like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to which the source index is being copied.
    * @param   sourceName
    *          The name of the index in the source table that is being copied 
    *          to the temp-table.
    * @param   dbTable
    *          The database table name from which to copy the index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addIndexLike(String name, String sourceName, String dbTable)
   {
      // TODO Auto-generated method stub
      return new logical();
   }
   
   /**
    * Adds to the temp-table an index like the one specified.
    * This method is the P2J equivalent of <code>ADD-LIKE-FIELD</code> method
    * of Progress 4GL.
    * 
    * @param   name
    *          The name of the index to which the source index is being copied.
    * @param   sourceName
    *          The name of the index in the source table that is being copied 
    *          to the temp-table.
    * @param   dbTable
    *          The database table name from which to copy the index.
    * 
    * @return  TBA: Doc are not explicit, most likely true on success
    */
   public logical addIndexLike(character name,
                               character sourceName,
                               character dbTable)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Signals that all the field and index definitions for a temp-table 
    * have been provided. After the call to this method no fields and indexes
    * can be added to this temporary table.
    * This method is the P2J equivalent of <code>TEMP-TABLE-PREPARE</code> 
    * method of Progress 4GL.
    * 
    * @param   name
    *          Temp-table name to be used in subsequent query statements that
    *          refer to this temp-table.
    *
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical tempTablePrepare(String name)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Signals that all the field and index definitions for a temp-table 
    * have been provided. After the call to this method no fields and indexes
    * can be added to this temporary table.
    * This method is the P2J equivalent of <code>TEMP-TABLE-PREPARE</code> 
    * method of Progress 4GL.
    * 
    * @param   name
    *          Temp-table name to be used in subsequent query statements that
    *          refer to this temp-table.
    *          
    * @return  TBA: Doc are not explicit, most likely true on success.
    */
   public logical tempTablePrepare(character name)
   {
      // TODO Auto-generated method stub
      return new logical();
   }

   /**
    * Obtain the handle to the buffer that was created by default for this 
    * temporary table. Every dynamic temp-table is created with at 
    * least one buffer. This buffer’s object handle is returned by this 
    * method. It cannot be called until the table has been prepared using 
    * <code>tempTablePrepare()</code> method.
    * This method is the P2J equivalent of <code>DEFAULT-BUFFER-HANDLE</code>
    * attribute.
    * 
    * @return  The default buffer handle for the temporary table.
    */
   public handle defaultBufferHandle()
   {
      // TODO Auto-generated method stub
      return new handle();
   }

   /**
    * Returns the state of the ERROR attribute
    *
    * @return    Current state of the ERROR attribute.
    */
   public logical isError()
   {
      // TODO:  implement
      return null;
   }

   /**
    * Setter for ERROR attribute.
    *
    * @param    value
    *           The new value of ERROR attribute.
    */
   public void setError(boolean value)
   {
      // TODO:  implement
   }

   /**
    * Setter for ERROR attribute.
    *
    * @param    value
    *           The new value of ERROR attribute.
    */
   public void setError(logical value)
   {
      // TODO:  implement
   }

}
